@@ -32,23 +32,28 @@ def execute(pagename, request):
     # does it.
     #
     form = request.form
-    userName = form['name'][0]
+    if not form.has_key('name'):
+        userName = None
+    else:
+	userName = form['name'][0]
     isCreate = (
         form.has_key('create') or
         form.has_key('create_only') or
         form.has_key('create_and_mail'))
-    isNewUser = not user.getUserId(request, userName)
+    isNewUser = userName and not user.getUserId(request, userName)
     message = userform.savedata(request)
     #
     # If that created a new user then generate a cookie and
     # see if he wants to send an email.
     #
-    u = user.User(request, auth_username=userName)
-    if isCreate and isNewUser and u.exists():
-        userform_action = wikiutil.importPlugin(
-                request.cfg, "macro", "EmailActivation",
-                function="userform_action")
-        new_message = userform_action(request, u)
-        if new_message != None:
-            message = new_message
+    if isCreate and isNewUser:
+	password = form['password'][0]
+	u = user.User(request, auth_username=userName, password=password)
+        if u.exists():
+	    userform_action = wikiutil.importPlugin(
+		    request.cfg, "macro", "EmailActivation",
+		    function="userform_action")
+	    new_message = userform_action(request, u)
+	    if new_message != None:
+		message = new_message
     Page.Page(request, pagename).send_page(request, msg=message)
