@@ -45,13 +45,31 @@ def action(pagename, request):
 # Userform action entry point.
 #
 def userform_action(request, u):
-    urandom_file = file("/dev/urandom")
-    u.emailactivation_cookie = "".join(
-            ["%02x" % ord(c) for c in urandom_file.read(8)])
+    #
+    # Generate the cookie, which should be completely unguessable.
+    # If there is a good source of random numbers use it.  Otherwise
+    # just use the random module.  The random module is completely
+    # deterministic - if you know the time the cookie was generated
+    # you can figure out what it is easily enough, so its simple to
+    # hack.
+    #
+    try:
+	urandom_file = file("/dev/urandom")
+	try:
+	    bytes = urandom_file.read(8)
+	finally:
+	    urandom_file.close()
+	u.emailactivation_cookie = "".join(["%02x" % ord(c) for c in bytes])
+    except EnvironmentError:
+	import random
+	random.seed()
+        u.emailactivation_cookie = "%013x" % random.randint(0, 1 << 52)
+    #
+    # Now send the email.
+    #
     url = "%s?n=%s&c=%s"  % (
             request.getBaseURL() + "/" + __name__.split(".")[-1],
             u.name, u.emailactivation_cookie)
-    urandom_file.close()
     email_data = emailActivation_email('create', request, u, url)
     message = None
     if email_data != None:
@@ -269,7 +287,7 @@ def emailActivation(request, form):
     if request.user.isSuperUser():
         return superEmailActivation(request, form)
     #
-    # Read in all user file.
+    # Read in all user files.
     #
     all_users = [
             user.User(request, id=userid)
@@ -301,13 +319,13 @@ def emailActivation(request, form):
     #
     url = request.getScriptname() + request.getPathinfo()
     pageVars = {
-            'action':           __name__.split(".")[-1],
-            'c':                wikiutil.escape(cookie),
-            'i':                wikiutil.escape(new_user.id),
-            'r':                wikiutil.escape(result),
-            'url':              url,
-            'user-email':       wikiutil.escape(getattr(new_user, 'email', '')),
-            'user-name':        wikiutil.escape(getattr(new_user, 'name', '')),
+            'action':	__name__.split(".")[-1],
+            'c':	wikiutil.escape(cookie),
+            'i':	wikiutil.escape(new_user.id),
+            'r':	wikiutil.escape(result),
+            'url':	url,
+            'user-email': wikiutil.escape(getattr(new_user, 'email', '')),
+            'user-name': wikiutil.escape(getattr(new_user, 'name', '')),
         }
     message = None
     redirect = ''
@@ -442,7 +460,7 @@ def superEmailActivation(request, form):
 
 #
 # Generate the email to send to a user.  Calls the user hook in the
-# configuration if there is one in inserts his customisations, using
+# configuration if there is one and inserts his customisations, using
 # the defaults where there aren't any.
 #
 ACTIVATED_EMAIL_TEXT = """
